@@ -238,6 +238,13 @@ NGeDSgam <- function(formula, family = "gaussian", data, weights = NULL, offset
     stop("`family' not recognized")
   }
   
+  # Ensure the response variable is a factor when using a binomial family
+  if (family$family == "binomial" && !is.factor(data[[response]])) {
+    data[[response]] <- as.factor(data[[response]])
+    if (nlevels(data[[response]]) != 2) 
+      stop("response is not a factor at two levels but ", sQuote("family = binomial"))
+  }
+  
   variance <- family$variance
   dev.resids <- family$dev.resids
   link <- family$linkfun # g
@@ -255,7 +262,7 @@ NGeDSgam <- function(formula, family = "gaussian", data, weights = NULL, offset
     "family" = family,
     "normalize_data" = normalize_data,
     "offset" = offset
-  )
+    )
   
   # Add 'response' as the first element of the list 'args'
   if (args$family$family != "binomial") {
@@ -286,6 +293,9 @@ NGeDSgam <- function(formula, family = "gaussian", data, weights = NULL, offset
       data[predictors] <- scale(data[predictors])
     }
   }
+  
+  # Boundary knots
+  args$extr <- lapply(data[ , sapply(data, is.numeric)], range)
  
   # Initialize the knots, intervals, and coefficients for each base-learner
   base_learners_list <- list()
