@@ -23,8 +23,54 @@
 #'
 #' @return Two data frames, \code{best_params} and \code{results}.
 #' \code{best_params} contains the best combination of parameters according to
-#' the cross-validated MSE. \code{results} presents the results for each of the
-#' possible combinations of parameters, given the input \code{parameters}.
+#' the cross-validated MSE. \code{results} presents the cross-validated MSE and 
+#' the average number of internal knots across the folds for each possible 
+#' combination of parameters, given the input \code{parameters}. In the case of
+#' \code{model_fun = NGeDSboost}, it also provides the cross-validated number of
+#' boosting iterations.
+#' 
+#' @examples
+#' ###################################################
+#' # Generate a data sample for the response variable
+#' # Y and the single covariate X
+#' set.seed(123)
+#' N <- 500
+#' f_1 <- function(x) (10*x/(1+100*x^2))*4+4
+#' X <- sort(runif(N, min = -2, max = 2))
+#' # Specify a model for the mean of Y to include only a component
+#' # non-linear in X, defined by the function f_1
+#' means <- f_1(X)
+#' # Add (Normal) noise to the mean of Y
+#' Y <- rnorm(N, means, sd = 0.1)
+#' data = data.frame(X = X, Y = Y)
+#' 
+#' \dontrun{
+#' ## NGeDS
+#' # Define different combinations of parameters to cross-validate
+#' param = list(beta_grid = c(0.5),
+#'              phi_grid = c(0.9, 0.95),
+#'              q_grid = c(2))
+#' 
+#' cv_NGeDS <- crossv_GeDS(Y ~ f(X), data = data, NGeDS, n = 3L,
+#'                         parameters = param)
+#' 
+#' print(cv_NGeDS$best_params)
+#' View(cv_NGeDS$results)
+#' 
+#' ## NGeDSboost
+#' param = list(int.knots_init_grid = c(1, 2),
+#'              shrinkage_grid = 1,
+#'              beta_grid = c(0.3, 0.5),
+#'              phi_grid = c(0.95, 0.99),
+#'              q_grid = 2)
+#' 
+#' cv_NGeDSboost <- crossv_GeDS(Y ~ f(X), data = data, NGeDSboost, n = 2L,
+#'                              n_folds = 2L, parameters = param)
+#' 
+#' print(cv_NGeDSboost$best_params)
+#' View(cv_NGeDSboost$results)
+#' }
+#' 
 #' @export
 #' @importFrom foreach foreach %dopar%
 #' @importFrom doParallel registerDoParallel
@@ -54,7 +100,7 @@ crossv_GeDS <- function(formula, data, model_fun, n = 2L, n_folds = 5L,
                                      n = n, n_folds = n_folds, parameters = parameters)
     }
   
-  return(list(best_params = cv$best_params, results = cv$results))
+  return(list(best_params = cv$best_params, results = as.data.frame(cv$results)))
 }
 
 ################################################################################
@@ -107,7 +153,7 @@ cross_validate.GeDS <- function(formula, data, model_fun, n = 2L, n_folds = 5L,
                            ncol = 11)
 
   # Register parallel backend
-  n_cores <- parallel::detectCores()
+  n_cores <- max(1, parallel::detectCores() - 1) # Avoid using all cores
   cl <- parallel::makeCluster(n_cores)
   doParallel::registerDoParallel(cl)
 
@@ -118,10 +164,9 @@ cross_validate.GeDS <- function(formula, data, model_fun, n = 2L, n_folds = 5L,
 
       # Initialize a vector to store MSE values for each fold
       mse_fold <- numeric(n_folds)
-      n_iterations_fold <- numeric(n_folds)
-      avg_intknots_fold <- list(avg_intknots_univbl = numeric(n_folds),
-                                avg_intknots_bivblX = numeric(n_folds),
-                                avg_intknots_bivblY = numeric(n_folds))
+      intknots_fold <- list(univbl = numeric(n_folds),
+                            bivblX = numeric(n_folds),
+                            bivblY = numeric(n_folds))
       time_fold <- numeric(n_folds)
 
       for (fold_idx in 1:n_folds) {
@@ -144,34 +189,49 @@ cross_validate.GeDS <- function(formula, data, model_fun, n = 2L, n_folds = 5L,
         end_time <- Sys.time()
         time_fold[fold_idx] <- difftime(end_time, start_time, units = "secs")
         # Evaluate the model on the test set
-        test_pred <- predict.GeDS(fit, newdata = test_data, n = n)
+        test_pred <- predict(fit, newdata = test_data, n = n)
 
         # Calculate the mean squared error for the test set
         mse_fold[fold_idx] <- mean((test_data[[outcome]] - test_pred)^2)
 
-        # Number of boosting iterations and average number of internal knots per boosting iteration
-        n_iterations_fold[fold_idx] <- fit$iters
-        if (fit$Type == "LM - Univ" || fit$Type == "GLM - Univ") {
-          avg_intknots_fold$avg_intknots_univbl[fold_idx] <- fit$Nintknots
-          avg_intknots_fold$avg_intknots_bivblX[fold_idx] <- NA
-          avg_intknots_fold$avg_intknots_bivblY[fold_idx] <- NA
-        } else if (fit$Type == "LM - Biv" || fit$Type == "GLM - Biv") {
-          avg_intknots_fold$avg_intknots_univbl[fold_idx] <- NA
-          avg_intknots_fold$avg_intknots_bivblX[fold_idx] <- fit$Nintknots$X
-          avg_intknots_fold$avg_intknots_bivblY[fold_idx] <- fit$Nintknots$Y
+        # Number of internal knots
+        if (inherits(fit, "GeDS")) {
+          if (fit$Type == "LM - Univ" || fit$Type == "GLM - Univ") {
+            intknots_fold$univbl[fold_idx] <- fit$Nintknots
+          } else if (fit$Type == "LM - Biv" || fit$Type == "GLM - Biv") {
+            intknots_fold$bivblX[fold_idx] <- fit$Nintknots$X
+            intknots_fold$bivblY[fold_idx] <- fit$Nintknots$Y
+          }
         }
 
       }
-
-      data.frame(beta = beta_grid[i],
-                 phi = phi_grid[j],
-                 q = q_grid[k],
-                 mse = mean(mse_fold),
-                 n_iterations = mean(n_iterations_fold),
-                 avg_intknots_univbl = mean(avg_intknots_fold$avg_intknots_univbl),
-                 avg_intknots_bivblX = mean(avg_intknots_fold$avg_intknots_bivblX),
-                 avg_intknots_bivblY = mean(avg_intknots_fold$avg_intknots_bivblY),
-                 avg_fitting_time = mean(time_fold))
+      
+      if (inherits(fit, "GeDS")) {
+        if (fit$Type == "LM - Univ" || fit$Type == "GLM - Univ") {
+          data.frame(beta = beta_grid[i],
+                     phi = phi_grid[j],
+                     q = q_grid[k],
+                     cv_mse = mean(mse_fold),
+                     avg_intknots = mean(intknots_fold$univbl),
+                     avg_fitting_time = mean(time_fold))
+          
+          } else if (fit$Type == "LM - Biv" || fit$Type == "GLM - Biv")
+            data.frame(beta = beta_grid[i],
+                       phi = phi_grid[j],
+                       q = q_grid[k],
+                       cv_mse = mean(mse_fold),
+                       avg_intknots_X = mean(intknots_fold$bivblX),
+                       avg_intknots_Y = mean(intknots_fold$bivblY),
+                       avg_fitting_time = mean(time_fold))
+        } else {
+          # NGeDSgam
+          data.frame(beta = beta_grid[i],
+                     phi = phi_grid[j],
+                     q = q_grid[k],
+                     cv_mse = mean(mse_fold),
+                     avg_fitting_time = mean(time_fold))
+        }
+      
     }
 
   # Stop the parallel backend
@@ -181,7 +241,7 @@ cross_validate.GeDS <- function(formula, data, model_fun, n = 2L, n_folds = 5L,
   results_matrix <- do.call(cbind, results_list)
 
   # Find the optimal parameters that minimize the cross-validated mse
-  best_params <- results_matrix[which.min(results_matrix[,"mse"]), ]
+  best_params <- results_matrix[which.min(results_matrix[,"cv_mse"]), ]
 
   # Return the optimal parameters
   return(list(best_params = best_params, results = results_matrix))
@@ -245,7 +305,7 @@ cross_validate.GeDSboost <- function(formula, data, model_fun, n = 2L, n_folds =
 
   # Parallel processing
   results_list <- foreach(i = 1:length(int.knots_init_grid), .combine = rbind, .packages = "GeDS",
-                          .export = c("avg_intknots")) %:%
+                          .export = c("n_intknots")) %:%
     foreach(j = 1:length(shrinkage_grid), .combine = rbind) %:%
     foreach(k = 1:length(beta_grid), .combine = rbind) %:%
     foreach(l = 1:length(phi_grid), .combine = rbind) %:%
@@ -254,9 +314,9 @@ cross_validate.GeDSboost <- function(formula, data, model_fun, n = 2L, n_folds =
       # Initialize a vector to store MSE values for each fold
       mse_fold <- numeric(n_folds)
       n_iterations_fold <- numeric(n_folds)
-      avg_intknots_fold <- list(avg_intknots_univbl = numeric(n_folds),
-                                avg_intknots_bivblX = numeric(n_folds),
-                                avg_intknots_bivblY = numeric(n_folds))
+      intknots_fold <- list(univbl = numeric(n_folds),
+                            bivblX = numeric(n_folds),
+                            bivblY = numeric(n_folds))
       time_fold <- numeric(n_folds)
 
       for (fold_idx in 1:n_folds) {
@@ -285,12 +345,12 @@ cross_validate.GeDSboost <- function(formula, data, model_fun, n = 2L, n_folds =
         # Calculate the mean squared error for the test set
         mse_fold[fold_idx] <- mean((test_data[[outcome]] - test_pred)^2)
 
-        # Number of boosting iterations and average number of internal knots per boosting iteration
-        aux <- avg_intknots(fit)
+        # Number of boosting iterations and total number of internal knots
+        aux <- n_intknots(fit, avg = FALSE)
         n_iterations_fold[fold_idx] <- aux$n_iterations
-        avg_intknots_fold$avg_intknots_univbl[fold_idx] <- aux$avg_intknots$avg_intknots_univbl
-        avg_intknots_fold$avg_intknots_bivblX[fold_idx] <- aux$avg_intknots$avg_intknots_bivblX
-        avg_intknots_fold$avg_intknots_bivblY[fold_idx] <- aux$avg_intknots$avg_intknots_bivblY
+        intknots_fold$univbl[fold_idx] <- aux$n_intknots$univbl
+        intknots_fold$bivblX[fold_idx] <- aux$n_intknots$bivblX
+        intknots_fold$bivblY[fold_idx] <- aux$n_intknots$bivblY
       }
 
       data.frame(int.knots_init = int.knots_init_grid[i],
@@ -298,11 +358,11 @@ cross_validate.GeDSboost <- function(formula, data, model_fun, n = 2L, n_folds =
                  beta = beta_grid[k],
                  phi = phi_grid[l],
                  q = q_grid[m],
-                 mse = mean(mse_fold),
+                 cv_mse = mean(mse_fold),
                  n_iterations = mean(n_iterations_fold),
-                 avg_intknots_univbl = mean(avg_intknots_fold$avg_intknots_univbl),
-                 avg_intknots_bivblX = mean(avg_intknots_fold$avg_intknots_bivblX),
-                 avg_intknots_bivblY = mean(avg_intknots_fold$avg_intknots_bivblY),
+                 avg_intknots_univbl = mean(intknots_fold$univbl),
+                 avg_intknots_bivblX = mean(intknots_fold$bivblX),
+                 avg_intknots_bivblY = mean(intknots_fold$bivblY),
                  avg_fitting_time = mean(time_fold))
     }
 
@@ -313,7 +373,7 @@ cross_validate.GeDSboost <- function(formula, data, model_fun, n = 2L, n_folds =
   results_matrix <- do.call(cbind, results_list)
 
   # Find the optimal internal_knots, max_iterations and shrinkage values that minimize the cross-validated mse
-  best_params <- results_matrix[which.min(results_matrix[,"mse"]), ]
+  best_params <- results_matrix[which.min(results_matrix[,"cv_mse"]), ]
 
   # Return the optimal internal_knots, max_iterations and shrinkage values
   return(list(best_params = best_params, results = results_matrix))
@@ -418,7 +478,7 @@ cross_validate.GeDSboost <- function(formula, data, model_fun, n = 2L, n_folds =
 #                   avg_intknots_fold$avg_intknots_bivblY[fold_idx] <- fit$Nintknots$Y
 #                 }
 #             } else if (inherits(fit, "GeDSgam")) {
-#               aux <- avg_intknots(fit)
+#               aux <- n_intknots(fit)
 #               n_iterations_fold[fold_idx] <- mean(fit$iters$backfitting)
 #             }
 #             
@@ -544,7 +604,7 @@ cross_validate.GeDSboost <- function(formula, data, model_fun, n = 2L, n_folds =
 #                 mse_fold[fold_idx] <- mean((test_data[[outcome]] - test_pred)^2)
 # 
 #                 # Number of boosting iterations and average number of internal knots per boosting iteration
-#                 aux <- avg_intknots(fit)
+#                 aux <- n_intknots(fit)
 #                 n_iterations_fold[fold_idx] <- aux$n_iterations
 #                 avg_intknots_fold$avg_intknots_univbl[fold_idx] <- aux$avg_intknots$avg_intknots_univbl
 #                 avg_intknots_fold$avg_intknots_bivblX[fold_idx] <- aux$avg_intknots$avg_intknots_bivblX
@@ -585,10 +645,10 @@ cross_validate.GeDSboost <- function(formula, data, model_fun, n = 2L, n_folds =
 # }
 
 
-################
-# avg_intknots #
-################
-avg_intknots <- function(object){
+##############
+# n_intknots #
+##############
+n_intknots <- function(object, avg = TRUE){
   # Compute the # of boosting iterations
   if (inherits(object, "GeDSboost")) {
     n_iterations <- if (object$args$initial_learner) object$iters + 1
@@ -599,31 +659,39 @@ avg_intknots <- function(object){
   n_intknots <- n_intknotsX <- n_intknotsY <- 0
   
   if (n_iterations == 0) {
-    avg_intknots_univbl <- avg_intknots_bivblX <- avg_intknots_bivblY <- 0
-    } else {
-      # Compute the average # of int. knots per boosting iteration
-      base_learners <- object$final_model$base_learners
-      for (bl_name in names(base_learners)) {
-        n_vars <- length(object$args$base_learners[[bl_name]]$variables)
-        if (n_vars == 1) {
-          n_intknots <- length(base_learners[[bl_name]]$linear.int.knots)
-          } else if (n_vars == 2) {
-            n_intknotsX <- length(base_learners[[bl_name]]$linear.int.knots$ikX)
-            n_intknotsY <- length(base_learners[[bl_name]]$linear.int.knots$ikY)
-          }
+    n_intknots_univbl <- n_intknots_bivblX <- n_intknots_bivblY <- 0
+  } else {
+    # Compute the average # of int. knots per boosting iteration
+    base_learners <- object$final_model$base_learners
+    for (bl_name in names(base_learners)) {
+      n_vars <- length(object$args$base_learners[[bl_name]]$variables)
+      if (n_vars == 1) {
+        n_intknots <- length(base_learners[[bl_name]]$linear.int.knots)
+      } else if (n_vars == 2) {
+        n_intknotsX <- length(base_learners[[bl_name]]$linear.int.knots$ikX)
+        n_intknotsY <- length(base_learners[[bl_name]]$linear.int.knots$ikY)
       }
-      avg_intknots_univbl <- n_intknots / n_iterations
-      avg_intknots_bivblX <- n_intknotsX / n_iterations
-      avg_intknots_bivblY <- n_intknotsY / n_iterations
     }
     
-    avg_intknots <- list(avg_intknots_univbl = avg_intknots_univbl,
-                         avg_intknots_bivblX = avg_intknots_bivblX,
-                         avg_intknots_bivblY = avg_intknots_bivblY)
+    if(avg) {
+      n_intknots_univbl <- n_intknots / n_iterations
+      n_intknots_bivblX <- n_intknotsX / n_iterations
+      n_intknots_bivblY <- n_intknotsY / n_iterations
+      } else {
+        n_intknots_univbl <- n_intknots
+        n_intknots_bivblX <- n_intknotsX 
+        n_intknots_bivblY <- n_intknotsY
+      }
     
+  }
+  
+  n_intknots <- list(univbl = n_intknots_univbl,
+                     bivblX = n_intknots_bivblX,
+                     bivblY = n_intknots_bivblY)
   
   
-  return(list(avg_intknots = avg_intknots, n_iterations = n_iterations))
+  
+  return(list(n_intknots = n_intknots, n_iterations = n_iterations))
 }
 
 ###################
